@@ -13,6 +13,7 @@
 #include "servicesdebug.h"
 #include <QDir>
 #include <QFile>
+#include <QScopedValueRollback>
 
 extern int servicesDebugArea();
 
@@ -22,6 +23,18 @@ KServiceFactory::KServiceFactory(KSycoca *db)
     , m_relNameDict(nullptr)
     , m_menuIdDict(nullptr)
 {
+    // Safety net against recursive calls. Because KSycocaFactory calls findFactory and that issues a checkDatabase
+    // and that may end up repairing the database and thus closing and reopening it, we must absolutely never find
+    // ourselves in a recursive call chain.
+    // The inner call would thrash the state of the outer call causing impossibly difficult to debug state corruption.
+    thread_local bool inside = false;
+    if (inside) {
+        qFatal(
+            "Recursive call detected in KServiceFactory. This is not allowed and indicates a bug in KSycoca/KServiceFactory. Please report this on "
+            "bugs.kde.org.");
+    }
+    QScopedValueRollback guard(inside, true);
+
     m_offerListOffset = 0;
     m_nameDictOffset = 0;
     m_relNameDictOffset = 0;
@@ -239,13 +252,13 @@ QList<KServiceOffer> KServiceFactory::offers(int serviceTypeOffset, int serviceO
 
     qint32 aServiceTypeOffset;
     qint32 aServiceOffset;
-    qint32 initialPreference;
+    qint32 offerPreference;
     qint32 mimeTypeInheritanceLevel;
     while (true) {
         (*str) >> aServiceTypeOffset;
         if (aServiceTypeOffset) {
             (*str) >> aServiceOffset;
-            (*str) >> initialPreference;
+            (*str) >> offerPreference;
             (*str) >> mimeTypeInheritanceLevel;
             if (aServiceTypeOffset == serviceTypeOffset) {
                 // Save stream position !
@@ -254,7 +267,7 @@ QList<KServiceOffer> KServiceFactory::offers(int serviceTypeOffset, int serviceO
                 KService *serv = createEntry(aServiceOffset);
                 if (serv) {
                     KService::Ptr servPtr(serv);
-                    list.append(KServiceOffer(servPtr, initialPreference, mimeTypeInheritanceLevel));
+                    list.append(KServiceOffer(servPtr, 1, mimeTypeInheritanceLevel));
                 }
                 // Restore position
                 str->device()->seek(savedPos);
@@ -278,13 +291,13 @@ KService::List KServiceFactory::serviceOffers(int serviceTypeOffset, int service
 
     qint32 aServiceTypeOffset;
     qint32 aServiceOffset;
-    qint32 initialPreference;
+    qint32 offerPreference;
     qint32 mimeTypeInheritanceLevel;
     while (true) {
         (*str) >> aServiceTypeOffset;
         if (aServiceTypeOffset) {
             (*str) >> aServiceOffset;
-            (*str) >> initialPreference; // unused (remove once KMimeTypeTrader/KServiceTypeTrader are gone)
+            (*str) >> offerPreference; // unused (remove once KMimeTypeTrader/KServiceTypeTrader are gone)
             (*str) >> mimeTypeInheritanceLevel; // unused (remove once KMimeTypeTrader/KServiceTypeTrader are gone)
             if (aServiceTypeOffset == serviceTypeOffset) {
                 // Save stream position !
@@ -317,13 +330,13 @@ bool KServiceFactory::hasOffer(int serviceTypeOffset, int serviceOffersOffset, i
     bool found = false;
     qint32 aServiceTypeOffset;
     qint32 aServiceOffset;
-    qint32 initialPreference;
+    qint32 offerPreference;
     qint32 mimeTypeInheritanceLevel;
     while (!found) {
         (*str) >> aServiceTypeOffset;
         if (aServiceTypeOffset) {
             (*str) >> aServiceOffset;
-            (*str) >> initialPreference;
+            (*str) >> offerPreference;
             (*str) >> mimeTypeInheritanceLevel;
             if (aServiceTypeOffset == serviceTypeOffset) {
                 if (aServiceOffset == testedServiceOffset) {
